# talKZ - (Kalfa and Zack)'s talk implementation
# Copyright (C) 2002:
# 	Cosimo Alfarano      <kalfa@cs.unibo.it>
# 	Stefano Zacchiroli   <zack@cs.unibo.it>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from time import time


import messages


class InvitesTable(object):
    """
    Invites table, used by talKZd to remember invites for users.

    Internal implementation use a list, namely self.__invites, of pairs
    <request, timestamp> sorted by timestamp, earlier reqeusts are toward the
    head of the list (element with index 0)
    """

    def __init__(self):
        self.__invites = []
        self.__nextId = 0
        self.__maxId = 16000    # see table.c from netkit-ntalk
        self.__MAX_LIFE = 60    # invites' expiration in seconds

    def add(self, req):
        """ add an invite to the table, return the id of the added invite (note
        that in general it is different than the id of the passed request) """

        if not isinstance(req, messages.Request):
            raise TypeError("'req' must be a Request object")
        req.setId(self.__getNextId())
        self.__invites.append((req, time()))
        return req.getId()

    def length(self):
        return len(self.__invites)

    def delete(self, id):
        self.__invites = \
            filter((lambda (req, _): req.getId() != id), self.__invites)

    def __getNextId(self):
        """ return next id upgrading internal status accordingly; successful
        invocations of this function will produce a non-zero id sequence """

        self.__nextId = (self.__nextId + 1) % self.__maxId
        if self.__nextId == 0:
            self.__nextId += 1
        return self.__nextId

    def newId():
        """ public interface to __getNextId """

        return self.__getNextId()

    def __expire(self):
        """ remove expired entries (those which are added more then
        self.__MAX_LIFE seconds ago) from invites list """

        self.__invites = filter(
                    (lambda (_, ts): (time() - ts) <= self.__MAX_LIFE),
                    self.__invites)

    def findRequest(self, req):
        """ find a request 'like' 'req'. Like is meant as having the same value
        for these fields: CallerName, CalleeName, Type, Pyd """

        self.__expire()
        try:
            for (cur_req, _) in self.__invites:
                if (req.getCallerName() == cur_req.getCallerName()) and \
                   (req.getCalleeName() == cur_req.getCalleeName()) and \
                   (req.getType() == cur_req.getType()) and \
                   (req.getPid() == cur_req.getPid()):
                       raise "Found", cur_req
            raise LookupError
        except "Found", item:
            return item

    def findPartner(self, req):
        """ find a partner matching request 'req' """

        self.__expire()
        try:
            for (cur_req, _) in self.__invites:
                if (req.getCallerName() == cur_req.getCalleeName()) and \
                   (req.getCalleeName() == cur_req.getCallerName()) and \
                   (cur_req.getType() == messages.LEAVE_INVITE):
                       raise "Found", cur_req
            raise LookupError
        except "Found", item:
            return item

