# talKZ - (Kalfa and Zack)'s talk implementation
# Copyright (C) 2002:
# 	Cosimo Alfarano      <kalfa@cs.unibo.it>
# 	Stefano Zacchiroli   <zack@cs.unibo.it>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import socket


class Peer(object):
    """ abstraction over a peer socket, ready to be used for talking; can hide
    a 'normal' socket or a ssl enabled one """

    def __init__(self, sock, ssl=False):
        if ssl not in [True, False]:
            raise TypeError("'ssl' must be a boolean")
        if not isinstance(sock, socket.socket):
            raise TypeError("'sock' must be a socket object")
        self.__ssl = ssl
        self.__fileno = sock.fileno()
        if self.isSSL():
            self.__sslobj = socket.ssl(sock)
        else:
            self.__sock = sock

    def isSSL(self):
        return self.__ssl

    def fileno(self):
        return self.__fileno

    def send(self, data):
        if type(data) != str:
            raise TypeError("'data' must be a string")
        if self.isSSL():
            bytes = self.__sslobj.write(data)
        else:
            bytes = self.__sock.send(data)
        return bytes

    def recv(self, size):
        if type(size) != int:
            raise TypeError("'size' must be an integer")
        if self.isSSL():
            data = self.__sslobj.read(size)
        else:
            data = self.__sock.recv(size)
        return data

